@@ -1,26 +1,28 @@
 /*
- *  Copyright (C) 2002 Christophe Fergeau
- *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004 Christian Persch
- *  Copyright (C) 2005 Tony Tsui
+ * Copyright (C) 2002 Christophe Fergeau
+ * Copyright (C) 2003, 2004 Marco Pesenti Gritti
+ * Copyright (C) 2003, 2004 Christian Persch
+ * Copyright (C) 2005 Tony Tsui
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ * This file is part of gnome-terminal.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * Gnome-terminal is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Gnome-terminal is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "terminal-notebook.h"
 #include "terminal-window.h"
+#include "terminal-intl.h"
 
 #include <glib-object.h>
 #include <gtk/gtk.h>
@@ -41,11 +43,13 @@ struct _TerminalNotebookPrivate
   gulong   toplevel_button_release_handler_id;
   gint     x_start, y_start;
   gboolean drag_in_progress;
-  GtkTooltips *title_tips;
+  GtkTooltips *tooltips;
+  GtkRcStyle *rcstyle;
 };
 
 static void terminal_notebook_init           (TerminalNotebook *notebook);
 static void terminal_notebook_class_init     (TerminalNotebookClass *klass);
+static void terminal_notebook_dispose        (GObject *object);
 static void terminal_notebook_finalize       (GObject *object);
 static void move_tab_to_another_notebook     (TerminalNotebook *src,
                                               TerminalNotebook *dest,
@@ -107,6 +111,7 @@ terminal_notebook_class_init (TerminalNotebookClass *klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  object_class->dispose = terminal_notebook_dispose;
   object_class->finalize = terminal_notebook_finalize;
 
   signals[TAB_ADDED] =
@@ -684,7 +689,13 @@ terminal_notebook_init (TerminalNotebook *notebook)
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
   gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
 
-  notebook->priv->title_tips = gtk_tooltips_new ();
+  notebook->priv->tooltips = gtk_tooltips_new ();
+  g_object_ref (notebook->priv->tooltips);
+  gtk_object_sink (GTK_OBJECT (notebook->priv->tooltips));
+
+  notebook->priv->rcstyle = gtk_rc_style_new ();
+  notebook->priv->rcstyle->xthickness = 0;
+  notebook->priv->rcstyle->ythickness = 0;
 
   g_signal_connect (notebook, "button-press-event",
                     (GCallback)button_press_cb, NULL);
@@ -694,6 +705,26 @@ terminal_notebook_init (TerminalNotebook *notebook)
 
 }
 
+static void
+terminal_notebook_dispose (GObject *object)
+{
+  TerminalNotebook *notebook = TERMINAL_NOTEBOOK (object);
+
+  if (notebook->priv->tooltips)
+    {
+      g_object_unref (notebook->priv->tooltips);
+      notebook->priv->tooltips = NULL;
+    }
+
+  if (notebook->priv->rcstyle)
+    {
+      g_object_unref (notebook->priv->rcstyle);
+      notebook->priv->rcstyle = NULL;
+    }
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 terminal_notebook_finalize (GObject *object)
 {
@@ -703,17 +734,26 @@ terminal_notebook_finalize (GObject *object)
 static void
 sync_label (TerminalScreen *screen, TerminalNotebook *nb)
 {
-  GtkWidget *ebox, *label;
+  GtkWidget *hbox, *ebox, *label;
+  GList *children;
   const char *title;
 
-  ebox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), GTK_WIDGET (screen));
-  label =
-    g_list_first (gtk_container_get_children (GTK_CONTAINER (ebox)))->data;
+  hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), GTK_WIDGET (screen));
+  children = gtk_container_get_children (GTK_CONTAINER (hbox));
+  ebox = g_list_first (children)->data;
+  g_list_free (children);
+  children = gtk_container_get_children (GTK_CONTAINER (ebox));
+  label = g_list_first (children)->data;
+  g_list_free (children);
 
   title = terminal_screen_get_title (screen);
 
   gtk_label_set_text (GTK_LABEL (label), title);
-  gtk_tooltips_set_tip (nb->priv->title_tips, ebox, title, NULL);
+
+  if (G_LIKELY (nb->priv->tooltips))
+    {
+      gtk_tooltips_set_tip (nb->priv->tooltips, ebox, title, NULL);
+    }
 }
 
 static void
@@ -721,8 +761,8 @@ close_button_clicked_cb (GtkWidget *widget, TerminalScreen *screen)
 {
   GtkWidget *notebook;
 
-  notebook = gtk_widget_get_parent (screen);
-  terminal_notebook_remove_tab (notebook, screen);
+  notebook = gtk_widget_get_parent (GTK_WIDGET (screen));
+  terminal_notebook_remove_tab (TERMINAL_NOTEBOOK (notebook), screen);
 }
 
 void
@@ -731,9 +771,10 @@ terminal_notebook_add_tab (TerminalNotebook *nb,
                            int position,
                            gboolean jump_to)
 {
-  gchar *title;
+  const char *title;
   GtkWidget *hbox, *label, *label_ebox, *close_button, *image;
-  GtkRcStyle *rcstyle;
+  GtkSettings *settings;
+  gint w, h;
 
   g_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
@@ -743,7 +784,10 @@ terminal_notebook_add_tab (TerminalNotebook *nb,
 
   label_ebox = gtk_event_box_new ();
   gtk_event_box_set_visible_window (GTK_EVENT_BOX (label_ebox), FALSE);
-  gtk_tooltips_set_tip (nb->priv->title_tips, label_ebox, title, NULL);
+  if (G_LIKELY (nb->priv->tooltips))
+    {
+      gtk_tooltips_set_tip (nb->priv->tooltips, label_ebox, title, NULL);
+    }
 
   label = gtk_label_new (title);
   gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
@@ -757,19 +801,28 @@ terminal_notebook_add_tab (TerminalNotebook *nb,
   gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
   gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
 
-  rcstyle = gtk_rc_style_new ();
-  rcstyle->xthickness = rcstyle->ythickness = 0;
-  gtk_widget_modify_style (close_button, rcstyle);
-  gtk_rc_style_unref (rcstyle);
+  if (G_LIKELY (nb->priv->rcstyle))
+    {
+      gtk_widget_modify_style (close_button, nb->priv->rcstyle);
+    }
+
+  settings = gtk_widget_get_settings (GTK_WIDGET (label));
+  gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU, &w, &h);
+  gtk_widget_set_size_request (close_button, w + 2, h + 2);
 
   image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (close_button), image);
   gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
+  if (G_LIKELY (nb->priv->tooltips))
+    {
+      gtk_tooltips_set_tip (nb->priv->tooltips, close_button, _("Close tab"), NULL);
+    }
+
   g_signal_connect (G_OBJECT (close_button), "clicked",
 		    G_CALLBACK (close_button_clicked_cb),
 		    screen);
-    
+
   gtk_widget_show (label);
   gtk_widget_show (label_ebox);
   gtk_widget_show (image);
